#!/bin/bash

LOGS_FOLDER="logs"
LOGS_FILE="logs.txt"
LOGGER_SCRIPT="logger.sh"
PROGNAME="user_management"
BASE_PATH="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

. $BASE_PATH/global_conf.config

source $BASE_PATH/$LOGGER_SCRIPT -l $LOGS_FOLDER$LOGS_FILE -f $PROGNAME -e "DEBUG"
OPTIND=1

source $BASE_PATH/appFramework.sh

OPTIND=1

function usage(){
    echo "createU -h for help, -u [username] for new user, -d [username] for db role";exit 0;
}


# Script arguments
 while getopts "n:" FLAG; do
   case $FLAG in
   n) workspaceName=$OPTARG;
   ;;
   \?) # Unknown option
      usage
   ;;
  esac
 done



# Main script logic
if [[ ! -z $workspaceName ]]; then

    workspaceName=$( echo "${workspaceName}" | awk '{print tolower($0)}' )
    workspaceFolder="/appl/wsp_"${workspaceName}
    workspaceType='poc'

    # not in production
    kinitForCreateOps

    # creates a workspace linux folder

    echo $workspaceFolder
    createWorkspaceLinuxFolder ${workspaceFolder} ${workspaceName} $workspaceType &&

    # creates workspace HDFS folder
    createWorkspaceHDFSFolder $workspaceFolder $workspaceName $workspaceType &&
    setQuota $workspaceFolder 500 &&
    createDatabase 'wsp_'$workspaceName'_poc' &&
    #creates a db role, but does not grant it to any group
    createDBrole $workspaceName 'y' &&
    grantOnDB 'wsp_'$workspaceName'_poc' $workspaceName &&
    grantOnURI $workspaceName $workspaceFolder || {
        ERRORT "Some operation has failed";
        exit 1;
    }
    INFOT "Script completed successfully"

else
    ERRORT "please enter workspace name as a script argument"

fi
