#!/bin/bash

# load config & functions
. configuration.ini
. appFramework.sh

DATE_FORMAT=$(date +"%Y-%m-%d-%H-%M-%S")
LOGS_FOLDER="logs/"
LOGGER_SCRIPT="./logger.sh"
PROGNAME="delete_workspace"
LOGGER_LOG_PATH="${LOGS_FOLDER}${PROGNAME}/${PROGNAME}-${DATE_FORMAT}"

skipErrors=0

# set params
while getopts ":n:j:s:" opt; do
    case "${opt}" in
        n) workspaceName="${OPTARG}";;
        s) skipErrors="${OPTARG}";;
    esac
done
OPTIND=1

if [[ ! -z "${workspaceName}" && ! -z "${jira}" ]]; then
    # check workspace name
    if [[ "${workspaceName}" == *['!'@#\$%^\&*()+/.] ]]; then
        ERRORTI "Workspace name contains forbidden character(s)"
        exit 1
    fi

    # kinit service_app_admin user
    . kinit.sh

    # init log functions and create log file
    source "${LOGGER_SCRIPT}" -l "${LOGGER_LOG_PATH}" -f "${PROGNAME}" -e "DEBUG"

    INFOT "Jira task: ${jira}"

    # 1) delete filesystem data
    # -------------------------
    dropUnixWspHomeDir "${workspaceName}" $skipErrors

    if [[ $? -eq 1 ]]; then
        ERRORT "Could not drop Unix project folder (${projectName})"

        if [[ $skipErrors -eq 0 ]]; then
            exit 1
        else
            INFOT "Skip error..."
        fi
    fi

    # 2) delete HDFS workspace home directory
    # ---------------------------------------
    deleteHdfsWspDir "${workspaceName}"

    if [[ $? -eq 1 ]]; then
        ERRORT "Could not drop HDFS workspace folder (${workspaceName})"

        if [[ $skipErrors -eq 0 ]]; then
            exit 1
        else
            INFOT "Skip error..."
        fi
    fi

    # 3) drop workspace database
    # --------------------------
    query="DROP DATABASE IF EXISTS ${workspaceName} CASCADE;"
    INFOT "query: ${query}"
    makeQuery "${query}" "hive"

    if [[ $? -eq 1 ]]; then
        ERRORT "Could not drop workspace database ${workspaceName}"

        if [[ $skipErrors -eq 0 ]]; then
            exit 1
        else
            INFOT "Skip error..."
        fi
    fi

    # 4) drop workspace roles
    # -----------------------
    query="DROP ROLE ${workspaceName}_write;" &&
    INFOT "query: ${query}"
    makeQuery "${query}" "hive" || {
        if [[ $skipErrors -eq 0 ]]; then
            exit 1
        else
            INFOT "Skip error..."
        fi
    }

    query="DROP ROLE ${workspaceName}_read;" &&
    INFOT "query: ${query}"
    makeQuery "${query}" "hive" || {
        if [[ $skipErrors -eq 0 ]]; then
            exit 1
        else
            INFOT "Skip error..."
        fi
    }

    kdestroy
    INFOT "Workspace ${workspaceName} has been deleted"
else
    ERRORTI "Not all parameters have been supplied."
    ERRORTI "Parameters are:"
    ERRORTI "-n Workspace name"
    ERRORTI "-j JIRA ticket"
    ERRORTI "-s Skip errors (1 = skip errors; default value = 0)"
fi
