#!/bin/bash

while getopts "l:f:e:" FLAG; do
 case $FLAG in
  l) SCRIPT_LOG=$OPTARG
     ;;
  f) FUNCTION=$OPTARG
     ;;
  e) LOG_LEVEL=$OPTARG
     ;;
  \?) #unrecognized option
     usage
     exit 1
     ;;
 esac
done
if [ $OPTIND -ne 7 ]; then echo "ERROR $(date +%H:%M:%S) `basename $0` - not enough parameters passed"; exit 1; fi
shift $((OPTIND-1))

DATE_FORMAT="%Y-%m-%d %H:%M:%S"
RED='\033[0;31m'
GREEN='\033[0;32m'
NC='\033[0m'
YELLOW='\033[0;33m'
LBLUE='\033[1;34m'

declare -A LOG_LEVELS
LOG_LEVELS=([DEBUG]=1 [INFO]=2 [WARN]=3 [ERROR]=4)
[ -z "${LOG_LEVELS[$LOG_LEVEL]}" ] && LOG_LEVEL=DEBUG

function SCRIPTENTRY(){
 script_name=$(basename $0)
 script_name=${script_name%.*}
 echo "$(date +"$DATE_FORMAT") DEBUG > $script_name $FUNCNAME" >> $SCRIPT_LOG
}

function SCRIPTEXIT(){
 script_name=$(basename $0)
 script_name=${script_name%.*}
 echo "$(date +"$DATE_FORMAT") DEBUG < $script_name $FUNCNAME" >> $SCRIPT_LOG
}

function ENTRY(){
 local cfn="${FUNCNAME[1]}"
 echo "$(date +"$DATE_FORMAT") DEBUG > $cfn $FUNCNAME" >> $SCRIPT_LOG
}

function EXIT(){
 local cfn="${FUNCNAME[1]}"
 echo "$(date +"$DATE_FORMAT") DEBUG < $cfn $FUNCNAME" >> $SCRIPT_LOG
}

function INFO(){
 (( "${LOG_LEVELS[$LOG_LEVEL]}" > "${LOG_LEVELS['INFO']}" )) && return 0
 funcname="${FUNCNAME[1]}"
 local function_name=${3:-$funcname}
 local msg="$1"
 local lineno="$2"
 echo -e "$(date +"$DATE_FORMAT")\tINFO\t${FUNCTION}\t${function_name}:$lineno\t$msg" >> $SCRIPT_LOG
}

function INFOT(){
 (( "${LOG_LEVELS[$LOG_LEVEL]}" > "${LOG_LEVELS['INFO']}" )) && return 0
 local msg="$1"
 local lineno="$2"
 echo -e "${GREEN}[INFO]${NC}\t$msg"
 INFO "$msg" "$lineno" "${FUNCNAME[1]}"
}

function DEBUG(){
 (( "${LOG_LEVELS[$LOG_LEVEL]}" > "${LOG_LEVELS['DEBUG']}" )) && return 0
 funcname="${FUNCNAME[1]}"
 local function_name=${3:-$funcname}
 local msg="$1"
 local lineno="$2"
 echo -e "$(date +"$DATE_FORMAT")\tDEBUG\t${FUNCTION}\t${function_name}:$lineno\t$msg" >> $SCRIPT_LOG
}

function DEBUGT(){
 (( "${LOG_LEVELS[$LOG_LEVEL]}" > "${LOG_LEVELS['DEBUG']}" )) && return 0
 local msg="$1"
 local lineno="$2"
 echo -e "${LBLUE}[DEBUG]${NC}\t$msg"
 DEBUG "$msg" "$lineno" "${FUNCNAME[1]}"
}

function ERROR(){
 funcname="${FUNCNAME[1]}"
 local function_name=${3:-$funcname}
 local msg="$1"
 local lineno="$2"
 echo -e "$(date +"$DATE_FORMAT")\tERROR\t${FUNCTION}\t${function_name}:$lineno\t$msg" >> $SCRIPT_LOG
}

function ERRORT(){
 local msg="$1"
 local lineno="$2"
 echo -e "${RED}[ERROR]${NC}\t$msg"
 ERROR "$msg" "$lineno" "${FUNCNAME[1]}"
}

function WARN(){
 (( "${LOG_LEVELS[$LOG_LEVEL]}" > "${LOG_LEVELS['WARN']}" )) && return 0
 funcname="${FUNCNAME[1]}"
 local function_name=${3:-$funcname}
 local msg="$1"
 local lineno="$2"
 echo -e "$(date +"$DATE_FORMAT")\tWARN\t${FUNCTION}\t${function_name}:$lineno\t$msg" >> $SCRIPT_LOG
}

function WARNT(){
 (( "${LOG_LEVELS[$LOG_LEVEL]}" > "${LOG_LEVELS['WARN']}" )) && return 0
 local msg="$1"
 local lineno="$2"
 echo -e "${YELLOW}[WARN]${NC}\t$msg"
 WARN "$msg" "$lineno" "${FUNCNAME[1]}"
}

touch $SCRIPT_LOG
