import MySQLdb
import logging
import configurationFile



class SqlQueries:
    def __init__(self):
        self.db = MySQLdb.connect(host=configurationFile.host, user=configurationFile.user,
                                  passwd=configurationFile.passwd, db=configurationFile.db)



    # Get Workspaces to check
    def get_group_workspaces_to_check(self):
        cursor = self.db.cursor()
        query = "select workspace_name from Workspace where ready_to_create = 0;"
        cursor.execute(query)
        result = cursor.fetchall()[0]
        return result

    # Get Workspace users
    def get_users_in_workspace(self, workspace):
        cursor = self.db.cursor()
        query = "select username from User u join Workspace_user wu on u.userID = wu.userID " \
                "join Workspace w on w.workspaceID = wu.workspaceID where w.workspace_name = '{}'".format(workspace);
        cursor.execute(query)
        gr = cursor.fetchall()
        users_in_workspace = [x[0] for x in gr]
        return users_in_workspace

    # Get workspace groups
    def get_workspace_groups(self, workspace):
        cursor = self.db.cursor()
        query = "select group_name from Linux_group lg join Workspace_group wg on lg.groupID = wg.groupID " \
                "join Workspace w on w.workspaceID = wg.workspaceID where workspace_name = '{}';".format(workspace)
        cursor.execute(query)
        gr = cursor.fetchall()
        wsp_groups = [x[0] for x in gr]
        return wsp_groups

    # Get users workspace groups
    def get_users_workspace_group(self, workspace, user):
        cursor = self.db.cursor()
        query = "select group_name from Linux_group lg join User_group ug on lg.groupID = ug.groupID " \
                 "join User u on u.userID = ug.userID join Workspace_user wu on wu.userID = u.userID " \
                 "join Workspace w on w.workspaceID = wu.workspaceID where w.workspace_name = '{}' " \
                 "and u.username = '{}';".format(workspace, user)
        cursor.execute(query)
        gr = cursor.fetchall()
        wsp_groups = [x[0] for x in gr]
        return wsp_groups

    # Set Workspace is ready
    def set_workspace_is_ready(self, workspace):
        cursor = self.db.cursor()
        query = "update Workspace set ready_to_create = 1 where workspace_name = '" + workspace + "';"

        try:
            cursor.execute(query)
            self.db.commit()
            return True
        except Exception as e:
            logging.exception(e)
            self.db.rollback()
            return False

    def close_connection(self):
        self.db.close()

    def get_prepared_workspaces(self):
        cursor = self.db.cursor()
        query = "select workspace_name from Workspace where ready_to_create = 1;"
        cursor.execute(query)
        gr = cursor.fetchall()
        wsps = [x[0] for x in gr]
        return wsps

    def set_group_is_checked_true(self, group):
        cursor = self.db.cursor()
        query = '''update linux_group set exists_on_all_nodes = 1 where group_name = "''' + group + '''";'''
        try:
            cursor.execute(query)
            self.db.commit()
        except Exception as e:
            logging.exception(e)
            self.db.rollback()

    def set_groups_are_checked_false(self, env):
        cursor = self.db.cursor()

        if env.lower() == "prod":
            query = '''update linux_group set exists_on_all_nodes_prod = 0 where group_name like "%_prod%";'''

        elif env.lower() == "test":
            query = '''update linux_group set exists_on_all_nodes_test = 0 where group_name like "%_test%";'''

        try:
            cursor.execute(query)
            self.db.commit()
        except Exception as e:
            logging.exception(e)
            self.db.rollback()

    def set_group_is_checked_true(self, group, env):
        cursor = self.db.cursor()

        if env.lower() == "prod":
            query = '''update linux_group set exists_on_all_nodes_prod = 1 where group_name = "''' + group + '''";'''

        elif env.lower() == "test":
            query = '''update linux_group set exists_on_all_nodes_test = 1 where group_name = "''' + group + '''";'''

        try:
            cursor.execute(query)
            self.db.commit()
        except Exception as e:
            logging.exception(e)
            self.rollback()

    def set_prod_user_is_checked_true(self, user):
        cursor = self.db.cursor()
        query = '''update role_based_access.user set exists_on_all_nodes_prod = 1 
        where username = "''' + user + '''";'''
        try:
            cursor.execute(query)
            self.db.commit()
        except Exception as e:
            logging.exception(e)
            self.db.rollback()

    def set_test_user_is_checked_true(self, user):
        cursor = self.db.cursor()
        query = '''update role_based_access.user set exists_on_all_nodes_test = 1 
        where test_username = "''' + user + '''";'''
        try:
            cursor.execute(query)
            self.db.commit()
        except Exception as e:
            logging.exception(e)
            self.db.rollback()

    def set_status_is_assigned(self, user, group, env):
        cursor = self.db.cursor()

        if env.lower() == "test":
            user_id = self.get_user_id(user, "test")
            group_id = self.get_group_id(group)
            query = "update user_group set status = 'Assigned' where groupID = " + str(group_id) + " and userID = '" + \
                    str(user_id) + "';"

        elif env.lower() == "prod":
            user_id = self.get_user_id(user, "prod")
            group_id = self.get_group_id(group)
            query = "update user_group set status = 'Assigned' where groupID = '" + str(group_id) + "' and userID = '" + \
                    str(user_id) + "';"

        try:
            cursor.execute(query)
            self.db.commit()
        except Exception as e:
            logging.exception(e)
            self.db.rollback()

    def set_system_is_integrated(self, system):
        cursor = self.db.cursor()
        query = "update source_system set is_integrated = 1 where system_name = " + system
        try:
            cursor.execute(query)
            self.db.commit()
            return True
        except Exception as e:
            logging.exception(e)
            self.db.rollback()
            return False

    def insert_into_user_group(self, user_id, group_id, status):
        cursor = self.db.cursor()
        query = "insert into user_group (userID, groupID, status) " \
                "values ('{0}','{1}', '{2}')".format(user_id, group_id, status)

        try:
            cursor.execute(query)
            self.db.commit()
            return True
        except Exception as e:
            logging.exception(e)
            self.db.rollback()
            return False

    def combination_exists(self, user_id, group_id, status):
        cursor = self.db.cursor()
        query = "select userID, groupID, status from user_group where userID = {0} and groupID = {1} and status = '{2}'"\
            .format(user_id, group_id, status)
        cursor.execute(query)
        row = cursor.fetchone()
        if row is None:
            return False
        else:
            return True

    def set_users_not_exist(self, env):
        cursor = self.db.cursor()

        if env.lower() == "prod":
            query = "update user set exists_on_all_nodes_prod = 0 where exists_on_all_nodes_prod = 1;"
        elif env.lower() == "test":
            query = "update user set exists_on_all_nodes_test = 0 where exists_on_all_nodes_test = 1;"

        cursor.execute(query)
        try:
            cursor.execute(query)
            self.db.commit()
            return True

        except Exception as e:
            logging.exception(e)
            self.db.rollback()
            return False

    def set_project_created_true(self, project):
        cursor = self.db.cursor()
        query = "update project set is_created = 1 where project_name = '" + project + "';"

        try:
            cursor.execute(query)
            self.db.commit()
            return True
        except Exception as e:
            logging.exception(e)
            self.db.rollback()
            return False

    def update_user_group_table(self, user_id, group_id, status):
        cursor = self.db.cursor()
        query = "update user_group set status = '" \
                + status + "' where userID = '" \
                + str(user_id) + "' and groupID = '" + str(group_id) + "';"
        try:
            cursor.execute(query)
            self.db.commit()
            return True
        except Exception as e:
            logging.exception(e)
            self.db.rollback()
            return False

    def set_system_groups_exist_on_all_nodes(self, system, env):
        cursor = self.db.cursor()
        if env.lower() == "prod":
            query = "update source_system set groups_exist_on_prod = 1 where system_name = " + system
        elif env.lower() == "test":
            query = "update source_system set groups_exist_on_test = 1 where system_name = " + system
        try:
            cursor.execute(query)
            self.db.commit()
            return True
        except Exception as e:
            logging.exception(e)
            self.db.rollback()
            return False

    def get_system_groups_status(self, group, env):
        cursor = self.db.cursor()
        query = "select exists_on_all_nodes_" + env + " from linux_group " \
                                                      "where group_name = '" + group + "';"
        try:
            cursor.execute(query)
            result = cursor.fetchall()[0]
        except IndexError:
            pass
        return result

    # Used in linux_check_skoda - workspace_is_ready

    def get_system_groups(self, system, env):
        cursor = self.db.cursor()
        query = "select group_name from linux_group lg " \
                    "join role_group rg on lg.id = rg.groupID " \
                    "join role r on r.id = rg.roleID " \
                    "join role_relation rr on r.id = rr.roleID " \
                    "join source_system ss on ss.id = rr.systemID " \
                    "where ss.system_name = '" + system + "' and group_name like '%" + env + "%';"
        cursor.execute(query)
        result = cursor.fetchall()[0]
        return result

    def get_pending_projects(self):
        cursor = self.db.cursor()
        query = "select project_name from project where is_created = 0;"
        cursor.execute(query)
        result = cursor.fetchall()[0]
        return result

    def get_project_users(self, project, env):
        cursor = self.db.cursor()
        if env.lower() == "prod":
            query = "select u.username from project p join user_project_role upr on p.id = upr.projectID " \
                    "join user u on u.id = upr.userID where project_name = '" + project + "' " \
                    " and u.username not like '%test%';"
        elif env.lower() == "test":
            query = "select u.test_username from project p join user_project_role upr on p.id = upr.projectID " \
                    "join user u on u.id = upr.userID where project_name = '" + project + "' " \
                    " and nullif(test_username, 'NULL') is not null"
        cursor.execute(query)
        result = cursor.fetchall()
        users = [x[0] for x in result]
        return users

    def get_user_groups_id_mariadb(self, user_id, env):
        cursor = self.db.cursor()

        if env.lower() == "prod":
            query = "select u.id, u.username, u.test_username, p.project_name, r.role_name, g.group_name, g.id " \
                    "from user u join user_project_role upr on u.id = upr.userID " \
                    "join project p on p.id = upr.projectID " \
                    "join role r on r.id = upr.roleID " \
                    "join role_group rg on rg.roleID = r.id " \
                    "join linux_group g on g.id = rg.groupID " \
                    "where u.id = '" + user_id + ";"

        elif env.lower() == "test":
            query = "select u.id, u.username, u.test_username, p.project_name, r.role_name, g.group_name, g.id " \
                    "from user u join user_project_role upr on u.id = upr.userID " \
                    "join project p on p.id = upr.projectID " \
                    "join role r on r.id = upr.roleID " \
                    "join role_group rg on rg.roleID = r.id " \
                    "join linux_group g on g.id = rg.groupID " \
                    "where u.test_username = '" + user_id + "' and g.group_name like '%test%';"

        cursor.execute(query)
        rows = cursor.fetchall()
        # print(rows)
        linux_groups_mariadb = []
        for row in rows:
            if row not in row:
                linux_groups_mariadb.append(row[6])
        # List of groups where specified user should have membership
        return linux_groups_mariadb

    def get_users(self, env):
        cursor = self.db.cursor()

        if env.lower() == "prod":
            query = "select username from user where username not like '%test%';"

        elif env.lower() == "test":
            query = "select test_username from user where nullif(test_username, 'NULL') is not null;"

        cursor.execute(query)
        result = cursor.fetchall()
        users = [x[0] for x in result]
        return users

    def get_user_groups_maria(self, user, env):
        cursor = self.db.cursor()
        query = "select distinct u.id, u.username, u.test_username, r.role_name, g.group_name " \
                    "from user u join role_relation rr on u.id = rr.userID " \
                    "join role r on r.id = rr.roleID " \
                    "join role_group rg on rg.roleID = r.id " \
                    "join linux_group g on g.id = rg.groupID " \
                    "where u.username = '" + user + "' and g.group_name like '%" + env + "%';"
        cursor.execute(query)
        rows = cursor.fetchall()
        # print(rows)
        linux_groups_maria = [i[4] for i in rows]
        # List of groups where specified user should have membership
        return linux_groups_maria

    def get_group_id_group(self, group, env):
        cursor = self.db.cursor()
        if env.lower() == "prod":
            query = "select id from linux_group where group_name = '" + group + "';"
        cursor.execute(query)
        group_id = cursor.fetchone()[0]
        return group_id

    def get_group_id(self, group):
        cursor = self.db.cursor()
        query = "select id from linux_group where group_name = '" + group + "';"
        cursor.execute(query)
        group_id = cursor.fetchone()[0]
        return group_id

    def get_user_id(self, user, env):
        cursor = self.db.cursor()
        if env.lower() == "prod":
            query = "select distinct id from user where username = '" + user + "';"
        elif env.lower() == "test":
            query = "select distinct id from user where test_username = '" + user + "';"

        cursor.execute(query)
        user_id = cursor.fetchone()[0]
        return user_id

    def get_not_checked_users_test(self):
        cursor = self.db.cursor()
        query = "select test_username from user where test_username != '' " \
                "and test_username is not null and exists_on_all_nodes_test = 0;"
        cursor.execute(query)
        result = cursor.fetchall()
        not_checked_users_test = [x[0] for x in result]
        return not_checked_users_test

    def get_not_checked_users_prod(self):
        cursor = self.db.cursor()
        query = "select username from user where username != '' " \
                "and username is not null and exists_on_all_nodes_prod = 0;"
        cursor.execute(query)
        result = cursor.fetchall()
        not_checked_users_prod = [x[0] for x in result]
        return not_checked_users_prod

    def get_groups(self, env):
        cursor = self.db.cursor()

        if env.lower() == "prod":
            query = "select group_name from linux_group where group_name like '%_prod%' " \
                    "and exists_on_all_nodes_prod = 0"

        elif env.lower() == "test":
            query = "select group_name from linux_group " \
                    "where group_name like '%_test%' and exists_on_all_nodes_test = 0;"
        cursor.execute(query)
        gr = cursor.fetchall()
        not_checked_groups = [x[0] for x in gr]
        return not_checked_groups



    def get_pending_systems(self):
        cursor = self.db.cursor()
        query = "select system_name from source_system where is_integrated = 0"
        cursor.execute(query)
        gr = cursor.fetchall()
        pending_systems = [x[0] for x in gr]
        return pending_systems

    def get_system_users(self, system, env):
        cursor = self.db.cursor()
        if env.lower() == "prod":
            query = "select username from source_system ss " \
                    "join role_relation rr on ss.id = rr.systemID " \
                    "join user u on u.id = rr.userID where system_name = '" + system + "';"
        elif env.lower() == "test":
            query = "select test_username from source_system ss " \
                    "join role_relation rr on ss.id = rr.systemID " \
                    "join user u on u.id = rr.userID where system_name = '" + system + \
                    "' and test_username is not null;"
        cursor.execute(query)
        gr = cursor.fetchall()
        pending_systems = [x[0] for x in gr]
        return pending_systems

    def get_user_status(self, user, env):
        cursor = self.db.cursor()
        query = "select exists_on_all_nodes_" + env + " from user where username = " + user + ";"
        cursor.execute(query)
        gr = cursor.fetchall()
        system_user_status = [x[0] for x in gr]
        return system_user_status

    def get_group_status(self, group, env):
        cursor = self.db.cursor()
        query = "select exists_on_all_nodes_" + env + " from linux_group where group_name = '" + group + "';"
        cursor.execute(query)
        gr = cursor.fetchall()
        group_status = [x[0] for x in gr]
        return group_status

    def get_system_users_group(self, user, system):
        cursor = self.db.cursor()
        query = "select username, group_name, system_name from user u " \
                "join role_relation rr on u.id = rr.userID " \
                "join source_system ss on rr.systemID = ss.id " \
                "join role r on r.id = rr.roleID join " \
                "role_group rg on rg.roleID = r.id " \
                "join linux_group lg on lg.id = rg.groupID " \
                "where system_name = '" + system + "' and username = '" + user + "';"
        cursor.execute(query)
        gr = cursor.fetchall()
        system_users_group = [x[1] for x in gr]
        return system_users_group

    def get_project_users_group(self, user, project):
        cursor = self.db.cursor()
        query = "select username, group_name, system_name from user u " \
                "join role_relation rr on u.id = rr.userID " \
                "join project p on rr.projectID = p.id " \
                "join role r on r.id = rr.roleID join " \
                "role_group rg on rg.roleID = r.id " \
                "join linux_group lg on lg.id = rg.groupID " \
                "where project_name = '" + project + "' and username = '" + user + "';"
        cursor.execute(query)
        gr = cursor.fetchall()
        project_users_group = [x[1] for x in gr]
        return project_users_group



    def get_group_assignment_status(self, group, user):
        cursor = self.db.cursor()
        query = "select status from user_group ug " \
                "join linux_group lg on ug.groupID = lg.id " \
                "join user u on u.id = ug.userID " \
                "where group_name = '" + group + "' " \
                "and username = '" + user + "';"
        cursor.execute(query)
        gr = cursor.fetchall()
        system_group_status = [x[0] for x in gr]
        return system_group_status

    def get_project_groups(self, project, env):
        cursor = self.db.cursor()
        query = "select group_name from linux_group lg " \
                "join role_group rg on lg.id = rg.groupID " \
                "join role r on r.id = rg.roleID " \
                "join role_relation rr on r.id = rr. roleID " \
                "join project p on p.id = rr.projectID " \
                "where project_name = '" + project + "';"
        cursor.execute(query)
        gr = cursor.fetchall()
        system_group_status = [x[0] for x in gr]
        return system_group_status
